package cordon.bleu.tour;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

@Controller
@RequestMapping("/tour/info")
public class TourInfoController {

    private TourService tourService;

    @Autowired
    public void setTourService(TourService tourService) {
        this.tourService = tourService;
    }

    @GetMapping("/{id}")
    public String show (@PathVariable int id, Model model) {
        Tour tour = tourService.getById(id);
        if (tour == null) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND,"Resource not found");
        }
        TourInfo tourInfo = tourService.getTourInfoById(id);
        if (tourInfo == null) {
            tourInfo = new TourInfo();
        }
        model.addAttribute("tourId", tour.getId());
        model.addAttribute("tourName", tour.getName());
        model.addAttribute("tourInfo", tourInfo);
        return "tour/info";
    }

    @GetMapping("/edit/{id}")
    public String edit(@PathVariable int id, Model model) {
        Tour tour = tourService.getById(id);
        if (tour == null) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND,"Resource not found");
        }
        TourInfo tourInfo = tourService.getTourInfoById(id);
        if (tourInfo == null) {
            tourInfo = new TourInfo();
            tourInfo.setId(id);
        }
        model.addAttribute("tourId", tour.getId());
        model.addAttribute("tourName", tour.getName());
        model.addAttribute("tourInfo", tourInfo);
        return "tour/info-edit";
    }


    @PostMapping("/edit/{id}")
    public String processEdit(@ModelAttribute TourInfo tourInfo, @PathVariable int id)  {
        Tour tour = tourService.getById(id);
        if (tour == null) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND,"Resource not found");
        }
        tourInfo.setId(id);
        tourInfo.setTour(tour);
        tourService.saveOrUpdate(tourInfo);
        return "redirect:/tour/info/" + id;
    }
}
