package cordon.bleu.tour;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Locale;

@Service
public class TourServiceImpl implements TourService{

    private TourRepository tourRepository;

    private TourInfoRepository tourInfoRepository;

    @Autowired
    public void setTourRepository(TourRepository tourRepository) {
        this.tourRepository = tourRepository;
    }

    @Autowired
    public void setTourInfoRepository(TourInfoRepository tourInfoRepository) {
        this.tourInfoRepository = tourInfoRepository;
    }

    @Override
    public List<Tour> getAll() {
        return tourRepository.findAll();
    }

    @Override
    public List<Tour> getNameContains(String needle) {
        return tourRepository.findByNameContains(needle);
    }

    @Override
    public Tour getById(int id) {
        return tourRepository.getTourById(id);
    }

    @Override
    public TourInfo getTourInfoById(int id) {
        return tourInfoRepository.getTourInfoById(id);
    }

    @Override
    public void saveOrUpdate(Tour tour) {
        tour.setCode(tour.getCode().toUpperCase());
        tourRepository.save(tour);
    }

    @Override
    public void saveOrUpdate(TourInfo tourInfo) {
        tourInfoRepository.save(tourInfo);
    }

    @Override
    public void delete(int id) {
        try {
            // delete tourInfo when available, this will also delete tour (cascade = All)
            tourInfoRepository.deleteById(id);
        } catch (Exception e) {
            // Delete tour only when no tourInfo exists
            tourRepository.deleteById(id);
        }
    }
}
